#include "slider.h"

namespace Msp {
namespace GLtk {

Slider::Slider():
	min(0),
	max(1),
	value(0),
	step(0.1),
	dragging(false),
	drag_area_size(0),
	drag_area_offset(0),
	slider_size(1)
{ }

void Slider::set_value(double v)
{
	double old_value = value;

	if(v<min)
		value = min;
	else if(v>max)
		value = max;
	else
	{
		unsigned steps = static_cast<unsigned>((v-min)/step+0.5);
		value = min+steps*step;
	}

	if(value!=old_value)
	{
		signal_value_changed.emit(value);
		rebuild();
	}
}

void Slider::set_range(double a, double b)
{
	min = a;
	max = b;
	set_value(value);
}

void Slider::set_step(double s)
{
	step = s;
	set_value(value);
}

void Slider::click(int p)
{
	p -= drag_area_offset;
	if(p<0 || p>=static_cast<int>(drag_area_size))
		return;

	int sp = static_cast<int>((drag_area_size-slider_size)*(value-min)/(max-min));
	if(p<sp)
		set_value(value-step*10);
	else if(p>=static_cast<int>(sp+slider_size))
		set_value(value+step*10);
	else
		start_drag(drag_area_offset+p);
}

void Slider::start_drag(int p)
{
	dragging = true;
	drag_start_pos = p;
	drag_start_value = value;
	set_state(ACTIVE);
}

void Slider::drag(int p)
{
	if(max>min)
		set_value(drag_start_value+(p-drag_start_pos)*(max-min)/(drag_area_size-slider_size));
}

void Slider::end_drag()
{
	dragging = false;
	clear_state(ACTIVE);
}


Slider::Loader::Loader(Slider &s):
	DataFile::DerivedObjectLoader<Slider, Widget::Loader>(s)
{
	add("range", &Slider::min, &Slider::max);
	add("step",  &Slider::step);
	add("value", &Slider::value);
}

} // namespace GLtk
} // namespace Msp
